@@ -1,14 +1,14 @@
 package HTML::Zoom;
 
-use strictures 1;
+use 5.008001;
+use strict;
+use warnings FATAL => 'all';
 
 use HTML::Zoom::ZConfig;
+use HTML::Zoom::MatchWithoutFilter;
 use HTML::Zoom::ReadFH;
-use HTML::Zoom::Transform;
-use HTML::Zoom::TransformBuilder;
-use Scalar::Util ();
 
-our $VERSION = '0.009008_01';
+our $VERSION = '0.009001';
 
 $VERSION = eval $VERSION;
 
@@ -29,16 +29,11 @@ sub _with {
   bless({ %{$_[0]}, %{$_[1]} }, ref($_[0]));
 }
 
-sub from_events {
-  my $self = shift->_self_or_new;
-  $self->_with({
-    initial_events => shift,
-  });
-}
-
 sub from_html {
   my $self = shift->_self_or_new;
-  $self->from_events($self->zconfig->parser->html_to_events($_[0]))
+  $self->_with({
+    initial_events => $self->zconfig->parser->html_to_events($_[0])
+  });
 }
 
 sub from_file {
@@ -53,7 +48,9 @@ sub to_stream {
     unless $self->{initial_events};
   my $sutils = $self->zconfig->stream_utils;
   my $stream = $sutils->stream_from_array(@{$self->{initial_events}});
-  $stream = $_->apply_to_stream($stream) for @{$self->{transforms}||[]};
+  foreach my $filter_spec (@{$self->{filters}||[]}) {
+    $stream = $sutils->wrap_with_filter($stream, @{$filter_spec});
+  }
   $stream
 }
 
@@ -61,14 +58,9 @@ sub to_fh {
   HTML::Zoom::ReadFH->from_zoom(shift);
 }
 
-sub to_events {
-  my $self = shift;
-  [ $self->zconfig->stream_utils->stream_to_array($self->to_stream) ];
-}
-
 sub run {
   my $self = shift;
-  $self->to_events;
+  $self->zconfig->stream_utils->stream_to_array($self->to_stream);
   return
 }
 
@@ -78,17 +70,6 @@ sub apply {
   $self->$code;
 }
 
-sub apply_if {
-  my ($self, $predicate, $code) = @_;
-  if($predicate) {
-    local $_ = $self;
-    $self->$code;
-  }
-  else {
-    $self;
-  }
-}
-
 sub to_html {
   my $self = shift;
   $self->zconfig->producer->html_from_stream($self->to_stream);
@@ -99,37 +80,22 @@ sub memoize {
   ref($self)->new($self)->from_html($self->to_html);
 }
 
-sub with_transform {
-  my $self = shift->_self_or_new;
-  my ($transform) = @_;
-  $self->_with({
-    transforms => [
-      @{$self->{transforms}||[]},
-      $transform
-    ]
-  });
-}
-  
 sub with_filter {
   my $self = shift->_self_or_new;
   my ($selector, $filter) = @_;
-  $self->with_transform(
-    HTML::Zoom::Transform->new({
-      zconfig => $self->zconfig,
-      selector => $selector,
-      filters => [ $filter ]
-    })
-  );
+  my $match = $self->parse_selector($selector);
+  $self->_with({
+    filters => [ @{$self->{filters}||[]}, [ $match, $filter ] ]
+  });
 }
 
 sub select {
   my $self = shift->_self_or_new;
   my ($selector) = @_;
-  return HTML::Zoom::TransformBuilder->new({
-    zconfig => $self->zconfig,
-    selector => $selector,
-    proto => $self
-  });
+  my $match = $self->parse_selector($selector);
+  return HTML::Zoom::MatchWithoutFilter->construct(
+    $self, $match, $self->zconfig->filter_builder,
+  );
 }
 
 # There's a bug waiting to happen here: if you do something like
@@ -145,37 +111,17 @@ sub select {
 
 sub then {
   my $self = shift;
-  die "Can't call ->then without a previous transform"
-    unless $self->{transforms};
-  $self->select($self->{transforms}->[-1]->selector);
+  die "Can't call ->then without a previous filter"
+    unless $self->{filters};
+  $self->select($self->{filters}->[-1][0]);
 }
 
-sub AUTOLOAD {
-  my ($self, $selector, @args) = @_;
-  my $sel = $self->select($selector);
-  my $meth = our $AUTOLOAD;
-  $meth =~ s/.*:://;
-  if (ref($selector) eq 'HASH') {
-    my $ret = $self;
-    $ret = $ret->_do($_, $meth, @{$selector->{$_}}) for keys %$selector;
-    $ret;
-  } else {
-    $self->_do($selector, $meth, @args);
-  }
+sub parse_selector {
+  my ($self, $selector) = @_;
+  return $selector if ref($selector); # already a match sub
+  $self->zconfig->selector_parser->parse_selector($selector);
 }
 
-sub _do {
-  my ($self, $selector, $meth, @args) = @_;
-  my $sel = $self->select($selector);
-  if( my $cr = $sel->_zconfig->filter_builder->can($meth)) {
-    return $sel->$meth(@args);
-  } else {
-    die "We can't do $meth on ->select('$selector')";
-  }
-}
-
-sub DESTROY {}
-
 1;
 
 =head1 NAME
@@ -213,14 +159,13 @@ HTML::Zoom - selector based streaming template engine
             $_->select('.name')->replace_content('Matt')
               ->select('.age')->replace_content('26')
           },
-          # alternate form
           sub {
-            $_->replace_content({'.name' => ['Mark'],'.age' => ['0x29'] })
+            $_->select('.name')->replace_content('Mark')
+              ->select('.age')->replace_content('0x29')
           },
-          #alternate alternate form
           sub {
-            $_->replace_content('.name' => 'Epitaph')
-              ->replace_content('.age' => '<redacted>')
+            $_->select('.name')->replace_content('Epitaph')
+              ->select('.age')->replace_content('<redacted>')
           },
         ],
         { repeat_between => '.between' }
@@ -340,15 +285,17 @@ cleanly:
     map { my $field = $_; sub {
 
      $_->select('label')
-       ->add_to_attribute( for => $field->{id} )
+       ->add_attribute( for => $field->{id} )
        ->then
        ->replace_content( $field->{label} )
-       ->add_to_attribute(
-        input => { 
-         name => $field->{name},
-         type => $field->{type},
-         value => $field->{value}
-       })
+
+       ->select('input')
+       ->add_attribute( name => $field->{name} )
+       ->then
+       ->add_attribute( type => $field->{type} )
+       ->then
+       ->add_attribute( value => $field->{value} )
+
     } } @fields
   ]);
 
@@ -402,7 +349,7 @@ HTML::Zoom does its best to defer doing anything until it's absolutely
 required. The only point at which it descends into state is when you force
 it to create a stream, directly by:
 
-  my $stream = $zoom->to_stream;
+  my $stream = $zoom->as_stream;
 
   while (my $evt = $stream->next) {
     # handle zoom event here
@@ -643,12 +590,6 @@ zoom instance with that as the source HTML to be transformed.
 
 Convenience method - slurps the contents of $file and calls from_html with it.
 
-=head2 from_events
-
-  my $zoom = HTML::Zoom->from_events($evt);
-
-Create a new Zoom object from collected events
-
 =head2 to_stream
 
   my $stream = $zoom->to_stream;
@@ -697,14 +638,6 @@ sugar, the following is entirely equivalent:
 
   my $z2 = $sub->($z1);
 
-=head2 apply_if
-
-  my $z2 = $z1->apply_if($cond, sub {
-    $_->select('div')->replace_content('I AM A DIV!') })
-  });
-
-->apply but will only run the tranform if $cond is true
-
 =head2 to_html
 
   my $html = $zoom->to_html;
@@ -742,7 +675,7 @@ In normal usage, you probably don't need to call this yourself.
 
   my $z2 = $z1->select('div')->replace_content('I AM A DIV!');
 
-Returns an intermediary object of the class L<HTML::Zoom::TransformBuilder>
+Returns an intermediary object of the class L<HTML::Zoom::MatchWithoutFilter>
 on which methods of your L<HTML::Zoom::FilterBuilder> object can be called.
 
 In normal usage you should generally always put the pair of method calls
@@ -750,56 +683,40 @@ together; the intermediary object isn't designed or expected to stick around.
 
 =head2 then
 
-  my $z2 = $z1->select('div')->add_to_attribute(class => 'spoon')
+  my $z2 = $z1->select('div')->add_attribute(class => 'spoon')
                              ->then
                              ->replace_content('I AM A DIV!');
 
 Re-runs the previous select to allow you to chain actions together on the
 same selector.
 
-=head1 AUTOLOAD METHODS
+=head2 parse_selector
 
-L<HTML::Zoom> AUTOLOADS methods against L</select> so that you can reduce a
-certain amount of boilerplate typing.  This allows you to replace:
+  my $matcher = $zoom->parse_selector('div');
 
-  $z->select('div')->replace_content("Hello World");
-  
-With:
+Used by L</select> and L</with_filter> to invoke the current
+L<HTML::Zoom::SelectorParser> object to create a matcher object (currently
+a coderef but this is an implementation detail) for that selector.
 
-  $z->replace_content(div => "Hello World");
-  
-Besides saving a few keys per invocations, you may feel this looks neater
-in your code and increases understanding.
+In normal usage, you probably don't need to call this yourself.
 
 =head1 AUTHOR
 
-mst - Matt S. Trout (cpan:MSTROUT) <mst@shadowcat.co.uk>
+Matt S. Trout (mst) <mst@shadowcat.co.uk>
 
-=head1 CONTRIBUTORS
+=head2 CONTRIBUTORS
 
-Oliver Charles
-
-Jakub Nareski
-
-Simon Elliott
-
-Joe Highton
-
-John Napiorkowski
-
-Robert Buels
-
-David Dorward
+None as yet, though I probably owe lots of people thanks for ideas. Yet
+another doc nit to fix.
 
 =head1 COPYRIGHT
 
-Copyright (c) 2010-2011 the HTML::Zoom L</AUTHOR> and L</CONTRIBUTORS>
+Copyright (c) 2009 - 2010 the HTML::Zoom L</AUTHOR> and L</CONTRIBUTORS>
 as listed above.
 
 =head1 LICENSE
 
-This library is free software, you can redistribute it and/or modify
-it under the same terms as Perl itself.
+This library is free software and may be distributed under the same terms
+as perl itself.
 
 =cut
-
