@@ -1,6 +1,7 @@
 package HTML::Zoom::Parser::BuiltIn;
 
-use strictures 1;
+use strict;
+use warnings FATAL => 'all';
 use base qw(HTML::Zoom::SubObject);
 
 sub html_to_events {
@@ -18,16 +19,12 @@ sub html_to_stream {
 
 sub _hacky_tag_parser {
   my ($text, $handler) = @_;
-  $text =~ m{^([^<]*)}g;
-  if ( length $1 ) { # leading PCDATA
-      $handler->({ type => 'TEXT', raw => $1 });
-  }
   while (
     $text =~ m{
       (
         (?:[^<]*) < (?:
             ( / )? ( [^/!<>\s"'=]+ )
-            ( (?:"[^"]*"|'[^']*'|[^/"'<>])+? )?
+            ( (?:"[^"]*"|'[^']*'|[^"'<>])+? )?
         |   
             (!-- .*? -- | ![^\-] .*? )
         ) (\s*/\s*)? >
@@ -35,31 +32,28 @@ sub _hacky_tag_parser {
       ([^<]*)
     }sxg
   ) {
-    my ($whole, $is_close, $tag_name, $attributes, $is_special,
+    my ($whole, $is_close, $tag_name, $attributes, $is_comment,
         $in_place_close, $content)
       = ($1, $2, $3, $4, $5, $6, $7, $8);
-    if ($is_special) {
-      $handler->({ type => 'SPECIAL', raw => $whole });
+    next if defined $is_comment;
+    $tag_name =~ tr/A-Z/a-z/;
+    if ($is_close) {
+      $handler->({ type => 'CLOSE', name => $tag_name, raw => $whole });
     } else {
-      $tag_name =~ tr/A-Z/a-z/;
-      if ($is_close) {
-        $handler->({ type => 'CLOSE', name => $tag_name, raw => $whole });
-      } else {
-        $attributes = '' if !defined($attributes) or $attributes =~ /^ +$/;
+      $attributes = '' if !defined($attributes) or $attributes =~ /^ +$/;
+      $handler->({
+        type => 'OPEN',
+        name => $tag_name,
+        is_in_place_close => $in_place_close,
+        _hacky_attribute_parser($attributes),
+        raw_attrs => $attributes||'',
+        raw => $whole,
+      });
+      if ($in_place_close) {
         $handler->({
-          type => 'OPEN',
-          name => $tag_name,
-          is_in_place_close => $in_place_close,
-          _hacky_attribute_parser($attributes),
-          raw_attrs => $attributes||'',
-          raw => $whole,
+          type => 'CLOSE', name => $tag_name, raw => '',
+          is_in_place_close => 1
         });
-        if ($in_place_close) {
-          $handler->({
-            type => 'CLOSE', name => $tag_name, raw => '',
-            is_in_place_close => 1
-          });
-        }
       }
     }
     if (length $content) {
